# database creation/privilege grant script for use with MySQL
# Tom Wheeler: For stlwebdev.org; Mon Jul 21 20:26:35 CDT 2003

# get rid of any previous database with the same name, and create the
# new one in its place (obviously you want to be careful with this).
drop database if exists addrbook;
create database addrbook;

# switch to the addrbook database
use addrbook;

# create the Contacts table.  This is an example application and is meant to 
# be simple and clear, but it works.  In real life, this could be optimized a lot.
create table Contacts 
(
	contactId           mediumint unsigned auto_increment primary key not null,
	firstName           varchar(30)                        null,
	lastName            varchar(30)                        null,
	addrLine1           varchar(50)                        null,
	addrLine2           varchar(50)                        null,
	city                varchar(50)                        null,
	state               varchar(2)                         null,
	zipCode             varchar(6)                         null,
	homePhone           varchar(12)                        null,
	workPhone           varchar(12)                        null,
	cellPhone           varchar(12)                        null,
	emailAddr           varchar(60)                        null,
	comments            text                               null
);

# set indexes on fields we think we will search often.  This will offer 
# improved search performance, but will slow down inserts and updates 
# so don't go overboard
create index idx_firstName on Contacts (firstName);
create index idx_lastName on Contacts (lastName);
create index idx_city on Contacts (city);
create index idx_state on Contacts (state);
create index idx_emailAddr on Contacts (emailAddr);

# set up the security on the database and table.  The servlet engine
# runs as user nobody, so we have to grant privileges to that account.
# this account doesn't need to create tables, alter tables, etc.
grant select, insert, update, delete
	on addrbook.*
	to nobody@localhost
	identified by 'mysecret';
