package com.tomwheeler.addrbook;

import com.tomwheeler.addrbook.ContactDetailForm;

// standard java classes
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Connection;
import java.sql.DriverManager;

// third-party classes
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
*  This is a trivial utility class that handles database access implementation
*  details in the Address Book application.  This is just a quick hack -- it is 
*  not intended to be a great example of how to do database access in Java, but
*  is instead meant to hide it so people concentrate on Struts and not JDBC. 
*
*@author     T. Wheeler (tom@stlwebdev.org)
*/
public class ContactDataUtil 
{    
    // set up the logger on instantiation
    private static Log logger = LogFactory.getLog(ContactDataUtil.class);
    
    /** Returns a connection to the database; in real life this would not be hardcoded! */
    protected Connection getConnection() throws SQLException
    {  
        logger.info("connection has been requested");
        String connUrl  = "jdbc:mysql://localhost:3306/addrbook?user=nobody&password=mysecret";
        try 
        {
            Class.forName("org.gjt.mm.mysql.Driver").newInstance();
        } 
        catch (Exception e)
        {
            logger.error("Caught an exception " + e.toString());
            throw new SQLException("Could not load database driver class");
        }
        Connection conn = DriverManager.getConnection(connUrl);
        logger.info("connection should be been returned");
        return conn;
    }    
    
    /**
    * This method will add the specified contact to the database, or throw
    * a SQLException if it cannot.
    *
    * @param  contactForm      The ContactDetailForm instance to be added to the database
    */
    public void addContact(
            ContactDetailForm contactForm) throws SQLException
    {
        logger.info("adding new contact");
        // get the updated properties from the ActionForm
        String firstName = contactForm.getFirstName();
        String lastName = contactForm.getLastName();
        String addrLine1 = contactForm.getAddrLine1();
        String addrLine2 = contactForm.getAddrLine2();
        String city = contactForm.getCity();
        String state = contactForm.getState();
        String zipCode = contactForm.getZipCode();
        String homePhone = contactForm.getHomePhone();
        String workPhone = contactForm.getWorkPhone();
        String cellPhone = contactForm.getCellPhone();
        String emailAddr = contactForm.getEmailAddr();
        String comments = contactForm.getComments();
        
        Connection conn = getConnection();

        StringBuffer sqlBuff = new StringBuffer();
        sqlBuff.append("insert into Contacts (firstName, lastName, addrLine1,  ");
        sqlBuff.append("addrLine2, city, state, zipCode, homePhone, workPhone,  ");
        sqlBuff.append("cellPhone, emailAddr, comments ) values (  ");
        sqlBuff.append("?,?,?,?,?,?,?,?,?,?,?,? )");
        
        logger.debug("adding new contact: SQL is :" + sqlBuff.toString());

        PreparedStatement stmt = conn.prepareStatement(sqlBuff.toString());

        stmt.setString(1, firstName);
        stmt.setString(2, lastName);
        stmt.setString(3, addrLine1);
        stmt.setString(4, addrLine2);            
        stmt.setString(5, city);
        stmt.setString(6, state);
        stmt.setString(7, zipCode);
        stmt.setString(8, homePhone);
        stmt.setString(9, workPhone);
        stmt.setString(10, cellPhone);
        stmt.setString(11, emailAddr);
        stmt.setString(12, comments);

        int rowsAffected = stmt.executeUpdate();
        logger.info("adding new contact: rows affected was " + rowsAffected);
        
        stmt.close();
        conn.close();
        logger.info("should have added new contact");
    }    

    /**
    * This method will update the specified contact to the database, or throw
    * a SQLException if it cannot.
    *
    * @param  contactForm The ContactDetailForm instance to be modified in the database
    */
    public void updateContact(
            ContactDetailForm contactForm) throws SQLException
    {
        logger.info("updating existing contact");
        
        // get the updated properties from the ActionForm
        String contactId = contactForm.getContactId();
        String firstName = contactForm.getFirstName();
        String lastName = contactForm.getLastName();
        String addrLine1 = contactForm.getAddrLine1();
        String addrLine2 = contactForm.getAddrLine2();
        String city = contactForm.getCity();
        String state = contactForm.getState();
        String zipCode = contactForm.getZipCode();
        String homePhone = contactForm.getHomePhone();
        String workPhone = contactForm.getWorkPhone();
        String cellPhone = contactForm.getCellPhone();
        String emailAddr = contactForm.getEmailAddr();
        String comments = contactForm.getComments();
        
        logger.info("updating existing contact : contactId is " + contactId);
        
        Connection conn = getConnection();

        StringBuffer sqlBuff = new StringBuffer();
        sqlBuff.append("update Contacts set firstName = ?, lastName = ?, ");
        sqlBuff.append("addrLine1 = ?, addrLine2 = ?, city = ?, state = ?, ");
        sqlBuff.append("zipCode = ?, homePhone = ?, workPhone = ?, cellPhone = ?,");
        sqlBuff.append("emailAddr = ?, comments = ? where contactId = ? ");

        PreparedStatement stmt = conn.prepareStatement(sqlBuff.toString());

        stmt.setString(1, firstName);
        stmt.setString(2, lastName);
        stmt.setString(3, addrLine1);
        stmt.setString(4, addrLine2);            
        stmt.setString(5, city);
        stmt.setString(6, state);
        stmt.setString(7, zipCode);
        stmt.setString(8, homePhone);
        stmt.setString(9, workPhone);
        stmt.setString(10, cellPhone);
        stmt.setString(11, emailAddr);
        stmt.setString(12, comments);
        stmt.setString(13, contactId);             

        int rowsAffected = stmt.executeUpdate();
        logger.info("updating existing contact: rows affected was " + rowsAffected);

        stmt.close();
        conn.close();     
        logger.info("should have updated contact details for contactId " + contactId);
    }

    /**
    * This method will remove the specified contact to the database, or throw
    * a SQLException if it cannot.
    *
    * @param  contactForm      The ContactDetailForm instance to be removed from the database
    */
    public void deleteContact(
            ContactDetailForm contactForm) throws SQLException
    {
        logger.info("deleting contact");
        
        // get the updated properties from the ActionForm
        String contactId = contactForm.getContactId();

        Connection conn = getConnection();

        StringBuffer sqlBuff = new StringBuffer();
        sqlBuff.append("delete from Contacts where contactId = ? ");
        PreparedStatement stmt = conn.prepareStatement(sqlBuff.toString());

        stmt.setString(1, contactId);
        logger.info("deleting contact with contactId: " + contactId);             

        int rowsAffected = stmt.executeUpdate();
        logger.info("deleting contact: rows affected was " + rowsAffected);

        logger.info("should have deleted contactId " + contactId);
        stmt.close();
        conn.close();           
    }
    
    
    /**
    * This method will return a populated ContactDetailForm for the specified contactId, 
    * or throw a SQLException if it cannot.
    *
    * @param  contactId The contactId which identifies the row to fetch from the database
    */
    public ContactDetailForm populateContactDetailForm (
            String contactId, ContactDetailForm contactForm) throws SQLException
    {
        logger.info("getting contact details");
        Connection conn = getConnection();

        StringBuffer sqlBuff = new StringBuffer();
        sqlBuff.append("select * from Contacts where contactId = ?");
        
        logger.info("getting contact details for contactId " + contactId);

        PreparedStatement stmt = conn.prepareStatement(sqlBuff.toString());

        // the id may be defined as an int in the table, but contactId is
        // defined as a String in this class;
        stmt.setString(1, contactId);            
        ResultSet rs = stmt.executeQuery();

        while (rs.next()) 
        {
            contactForm.setFirstName(rs.getString("firstName"));
            contactForm.setLastName(rs.getString("lastName"));
            contactForm.setAddrLine1(rs.getString("addrLine1"));
            contactForm.setAddrLine2(rs.getString("addrLine2"));
            contactForm.setCity(rs.getString("city"));
            contactForm.setState(rs.getString("state"));
            contactForm.setZipCode(rs.getString("zipCode"));
            contactForm.setHomePhone(rs.getString("homePhone"));
            contactForm.setWorkPhone(rs.getString("workPhone"));
            contactForm.setCellPhone(rs.getString("cellPhone"));
            contactForm.setEmailAddr(rs.getString("emailAddr"));
            contactForm.setComments(rs.getString("comments"));
        }

        stmt.close();
        conn.close();
        
        logger.info("should return contact details for contactId " + contactId);
        return contactForm;        
    }    
}